
/*These imports are for the VRML nodes, etc etc. */
import vrml.*; 
import vrml.field.*; 
import vrml.node.*;

/* Java Specific classes */
import java.util.*; 
import java.net.*; 
import java.io.*;

/*The IIOP-Collab specific part */
import Client.*;
import MultiCoordinator.*;
import PartyScheduler.*;

/*This is a porting of the initial prototype of a collaboratory VRML, world
  by Balaji Natarajan & Shrideep Pallickara
   @version     Alpha 1.0 22nd October 1997
   @author      Shrideep Pallickara / Balaji Natrajan
   @author      NorthEast Parallel Architectures Center
 */

public class
CVRUser extends Script implements MuProtocol, Runnable {
  /*The MultiCoordinator contains handles to all the Clients within a certain 
    Party */
  public MultiCoordinator.Coordinator myCoordinator;

  /*The Party Scheduler is the dude which maintains all the Party Handles */
  public PartyScheduler.PartyCoordinator partyCoordinator;

  /** Info pertinent to the Client. */
  public String clientName;
  public int clientID;

  /**Info pertinent to the Party*/
  public String partyName="Wojtek";

  /*The party Handle is always received against the partyID */
  public int partyID;
  
  /** Indicates if the client is successfully connected to the server. */
  boolean connected = false;
  
  private Thread running = null;
  private Thread controlThread;
  public  String clientNumber="1";


  
  /*Information pertinent to modifying the VRML world */
  public final static int AVATARS = 3; 
  Browser b;                     // for displaying error message to user 
  SFVec3f avatarPosition[] = new SFVec3f[AVATARS];          
  SFRotation avatarRotation[] = new SFRotation[AVATARS]; 
  float[] coord = new float[3]; 
  float[] rotation; 
  int id; 
  
  String connect_info = 0+ " " + 0 +" " + 0;
  int [] information = {0,0,0};
  
  public void initialize(){ 
    b = getBrowser(); 
    Node avatar = null; 
    
    /* Get the reference to 'translation' and 'rotation' field
       of 'avatar' node */

    for (int i=0; i < AVATARS; i++){ 
      try { 
	avatar = (Node)((SFNode)getField("avatar" + i)).getValue(); 
	avatarPosition[i] = (SFVec3f)avatar.getExposedField("translation"); 
	avatarRotation[i] = (SFRotation)avatar.getExposedField("rotation"); 
      } catch (Exception e) { 
	b.setDescription("can not get avatar"); 
	return; 
      } 
    } 
    rotation = new float[4]; 
  }

  private boolean isConnected() {
    System.out.println("ChatUser ===>>>Connect is " + connected);
    return(connected);
  }

  private boolean connect() {
    controlThread = new VRMLControlThread(this);
    controlThread.start();
    return true;
  }
    
  private void binder() {
    /* Initialize the ORB */
    org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init();

    /* Calculate Start time */
    long startTime = System.currentTimeMillis();

    /* Bind to the PartyCoordinator Object */
    partyCoordinator = PartyScheduler.PartyCoordinatorHelper.bind(orb);

    /* Calculate stop time; print out statistics
       This is included for diagonistic purposes */
    long stopTime = System.currentTimeMillis();
    System.out.println("Avg Ping to bind = "
		       + ((stopTime - startTime)/1000f) + " secs");
    System.out.println("Party Scheduler is Ready");
  }
  
  private void switchToCollaborationMode() {

    /*If the party doesn't exist this control statement results in
      a party being created else its joins a party */
    if (partyCoordinator.createParty(partyName))
      System.out.println("New Party created");
    else 
      System.out.println("Party being joined");

    /*Get the partyID as a precursor to receiving the party Handle */
    partyID= partyCoordinator.getPartyID(partyName);
    System.out.println("Party Id is: "+partyID);

    /*Receive the Party Handle */
    myCoordinator = partyCoordinator.getPartyHandle(partyID);
    System.out.println("Received Party Handle");
    
    /*Getting your TVR ID dude */
    id = myCoordinator.numberOfMembers();

    /*Publish your Object Reference to the ORB */
    connected= connect();
    System.out.println("READY TO MERINGUE");
  }
  
  public boolean broadcast(String message) {
    return myCoordinator.broadcast(message);
  }

  public boolean dataReceive(String commandLine) {
    StringTokenizer tok;
    float x=0f, y=0f, z=0f, r=0f;    
    
    while (commandLine != null) {
      tok = new StringTokenizer(commandLine, " ,\n\r\t");
      commandLine = null;
      int idOfMove      = Integer.parseInt(tok.nextToken());
      int command = Integer.parseInt(tok.nextToken());
      
      x = Float.valueOf(tok.nextToken()).floatValue();
      y = Float.valueOf(tok.nextToken()).floatValue();
      z = Float.valueOf(tok.nextToken()).floatValue();
      
      if(MuProtocol.MOVE == command){
	updatePosition(idOfMove, x, y, z); // update position 
      }else{
	r = Float.valueOf(tok.nextToken()).floatValue();
	updateOrientation(id, x, y, z, r); // update orientation
      }
    }
    return true;
  }
  
   private void disconnect() {
     System.out.println("Deregistring Client ID" +clientID + clientName);
     myCoordinator.deregister(clientID);
     System.out.println("Deregistring complete ... Listing follows");

     /*From this part on the processing is purely for
       statistical purposes */
     int numberOfMembers= myCoordinator.numberOfMembers();
     String[] temp = new String[numberOfMembers];
     System.arraycopy(myCoordinator.getClientNames(),0, 
		      temp,0,numberOfMembers);
     for (int i=0;i< numberOfMembers; i++)
       System.out.println(temp[i]);
     System.out.println("Listing complete");
   }


   public void stop() {
     if (running != null) {
       running.stop();
       running = null;
     }
   }    
   
   
   public void run() {}
   
   public void destroy() {
     System.out.println("destroyed session");
   }

   /*We could need this function for some random generations later on */
   private int rrange(int low, int high) {
     return((int)(Math.random() * (high - low + 1) + low));
   }


   public void processEvent(Event ev) { 
     if(ev.getName().equals("clicked1") == true){  
       b.setDescription("Event Name is "+ev.getName());
       ConstSFBool v = (ConstSFBool)ev.getValue(); 
       
       if(v.getValue() && (!connected)){ 
	 clientName = "ShriBala"+ ":" +
	   (rrange(0, 1000) * rrange(0, 1000));
	 switchToCollaborationMode();
	 b.setDescription("Connection to server established"); 
	 return; 
       }
       else if(ev.getName().equals("currentPosition")){ 
	 ((ConstSFVec3f)ev.getValue()).getValue(coord);  
	 //coord[0]=0f; coord[1]=0f; coord[2]=0f; 
	 System.out.println("x:" + coord[0] + "y:" + coord[1] + 
			    "z:" + coord[2] );
	 b.setDescription("Event Name is "+ev.getName()); 
	 
	 if (connected) {
	   try { 
	     // send my id and my current position to server 
	     broadcast(id + " " + MuProtocol.MOVE + " " +  coord[0] + 
		       " " + coord[1] + " " + coord[2] ); 
	   } catch (Exception e) { 
	     System.err.println("Caught exception while myPosition: " + e);
	   } 
	 }
       } 
       else if(ev.getName().equals("myOrientation")){ 
	 ((ConstSFRotation)ev.getValue()).getValue(rotation); 
	 try { 
	   // send my id and my current position to server 
	   broadcast(id + " " + MuProtocol.ROTATION + " " +  rotation[0] + 
		     " " + rotation[1] + " " + rotation[2] +
		     " " + rotation[3]); 
	 } catch (Exception e) { 
	   System.err.println("Caught exception while invoke myOrient: " + e);
	 } 
       }
     } 
   }
     
   public void updatePosition(int id, float x, float y, float z){  
     System.out.println("Avatar" + id +"on the move ....");
     avatarPosition[id].setValue(x, y, z); 
   } 
   
   public void updateOrientation(int id, float x, float y, float z, float r){ 
     avatarRotation[id].setValue(x, y, z, r); 
   }
   
}

/*This thread is responsible for the callbacks from the Server side */

class VRMLControlThread extends Thread {
  private CVRUser thisClient;
  private MultiCoordinator.Coordinator myCoordinator;
  private com.visigenic.vbroker.IOP.IORHelper remote_ior_file; 
  private PartyScheduler.PartyCoordinator partyCoordinator;
  VRMLControlThread(CVRUser client)
  { thisClient = client;
  }
  
  public void run()
  { try
    { // Initialize the ORB.
      org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init();
      
      // Initialize the BOA.
      org.omg.CORBA.BOA boa = orb.BOA_init();
      
      
      // Create the ClientControl object.
      VRControlImpl control =
	new VRControlImpl(thisClient);
      
      // Export the newly created object.
      boa.obj_is_ready(control);      
      System.out.println("Registering ......");
      // Register with Coordinator
      
      int id= thisClient.myCoordinator.register(thisClient.clientNumber.hashCode(), thisClient.clientName,(Client.ClientControl) control);
      thisClient.clientID=id;
      if (id < 20)  System.out.println("Registration Succeeded");
      else  System.out.println("Registration Failed");
      
      // Ready to service requests.
      System.out.println("Control: Waiting for requests");
      //boa.impl_is_ready();
    }
  catch(org.omg.CORBA.SystemException e)
    { System.err.println(e);
    }
  }
}
   
   

