import java.awt.*;
import java.applet.Applet;


/**<i>
  <b>
 * This is just a simple server/client based proof-of-concept
 * implementation to make sure that the compensation classes have
 * all the required API in them. </b></i>
 *
 * @version     Alpha 1.0 6th October 1997
 * @author      shrideep@npac.syr.edu
 * @author      Shrideep Pallickara/ Wojtek Furmanski
 * @author      NorthEast Parallel Architectures Center
 */

/*Below is the Chat User program. The applet consists of three parts:
 *a means of setting a name for the user
 *a scrolling list of messages
 *a means of sending a new message
 */

public class
ChatUser extends Applet implements  Runnable {
    Button    setIDButton;
    Button    setNameButton;
    Button    sayButton;
    Button    signOffButton;
    Button partyNameButton;
    TextField idField;
    TextField nameField;
    TextField typeField;
    TextField partyField;
    TextArea  messageArea;
    public MultiCoordinator.Coordinator myCoordinator;
    public PartyScheduler.PartyCoordinator partyCoordinator;

    /** The name of this Client. */
    public String clientName;
    public int clientID;
    /**The name of the Party*/
    public String partyName;
    public int partyID;

    /** Indicates if the client is successfully connected to the server. */
    boolean connected = false;

    /* Default setup, will mostly be overriden by attributes. */

    int     width       = 250;
    int     height      = 250;

    private Thread running = null;
    private Thread controlThread;
    public  String clientNumber="1";

    public synchronized void
    init() {
        Panel p;
        Font               lbl;
        GridBagLayout      g;
        GridBagConstraints c;

	System.out.println("ChatUser: init.");


	/* Get needed attributes from the html page. */

        if (getParameter("width") != null) {
            width = Integer.parseInt(getParameter("width"));
        } else {
            width = 250;
        }

        if (getParameter("height") != null) {
            height = Integer.parseInt(getParameter("height"));
        } else {
            height = 250;
        }

        lbl = new Font("Helvetica", Font.BOLD, 14);
        g   = new GridBagLayout();
        c   = new GridBagConstraints();

	c.fill      = GridBagConstraints.NONE;
	c.weightx   = 1.0;
	c.gridwidth = GridBagConstraints.REMAINDER;
 
	setLayout(g);
 
	p = new Panel();
	g.setConstraints(p, c);
	add(p);
	p.add(setIDButton = new Button("SetID"));
	p.add(idField     = new TextField(2)).setFont(lbl);
	p.add(setNameButton = new Button("Set Name"));
	p.add(nameField     = new TextField(20)).setFont(lbl);

	p = new Panel();
	g.setConstraints(p, c);
	add(p);
	p.add(partyNameButton = new Button("Party"));
	p.add(partyField     = new TextField(20)).setFont(lbl);
	p.add(signOffButton = new Button("SignOff"));
	signOffButton.disable();

	p = new Panel();
	g.setConstraints(p, c);
	add(p);
	p.add(new Label("Messages: "));
	p.add(messageArea = new TextArea(6, 30));
 
	p = new Panel();
	g.setConstraints(p, c);
	add(p);
	p.add(sayButton = new Button("Say"));
	sayButton.disable();
	p.add(typeField = new TextField(20));
        resize(width, height);
        show();
    }


    private boolean
    isConnected() {
      System.out.println("ChatUser: isConnected: " + connected);
      return(connected);
    }


    private boolean
    connect() {
      controlThread = new ClientControlThread(this);
      System.out.println("Just before starting ClientControl Thread...");
      controlThread.start();
      return true;
    }
    
    private void binder() {
      // Initialize the ORB.
      org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(this);
      // Calculate Start time
      long startTime = System.currentTimeMillis();
      // Bind to the Coordinator Object
      //myCoordinator = MultiCoordinator.CoordinatorHelper.bind(orb);
      partyCoordinator = PartyScheduler.PartyCoordinatorHelper.bind(orb);
      // Calculate stop time; print out statistics
      long stopTime = System.currentTimeMillis();
      System.out.println("Avg Ping to bind = "
			 + ((stopTime - startTime)/1000f) + " msecs");
      System.out.println("Party Scheduler is Ready");
    }
    
    /*  Convert the users message to bytes and send it over the channel. */
    
    public boolean
      broadcast(String message) {
      return myCoordinator.broadcast(message);
     
    }
    
    public boolean dataReceive(String Message) {
      int     position;
      String message=  Message + "\n";
      position = messageArea.getText().length();
      messageArea.insertText(message, position);
      return true;
    }

    private void
    disconnect() {
      System.out.println("Deregistring Client ID" +clientID + clientName);
      myCoordinator.deregister(clientID);
      System.out.println("Deregistring complete ... Listing follows");
      int numberOfMembers= myCoordinator.numberOfMembers();
      String[] temp = new String[numberOfMembers];
      System.arraycopy(myCoordinator.getClientNames(),0, temp,0,numberOfMembers);
      for (int i=0;i< numberOfMembers; i++)
	System.out.println(temp[i]);
      System.out.println("Listing complete");
    }


    public void
    start() {
      System.out.println("ChatUser: start.");
      if (running == null) {
	running = new Thread(this);
	running.start();
      }
    }
    

    public void
    stop() {
      if (running != null) {
	running.stop();
	running = null;
      }
    }    
    
    
    public void
      run() {
    }
    
    
    public void
      destroy() {
      System.out.println("destroyed session");
    }

    /*The first thing to do is to set a name for the user. Type the users name
     in the upper text field, then click the "Set Name" button. The text field
     is grayed out, and the "Set Name" button is inactivated.

     You have now joined the chat session. You can send messages by typing them
     in the bottom text field and when you are ready to send them, click on the
     "Say" button. The message will be sent to all users joined to the chat
     session. It will appear as the last entry in the scrolling message list.

     */


    public boolean
    action(Event event, Object obj) {
      System.out.println("ChatUser: action.");
     
	if ("SignOff".equals(obj)) {
	    sayButton.disable();
	    signOffButton.disable();
	    setNameButton.disable();
	    disconnect();
	    return(true);
	} else if ("Say".equals(obj)) {

	  // Say the message field info to all the players
	  
	  return broadcast(clientName +":"+typeField.getText());
	  
	} else if ("SetID".equals(obj)) {
	  /*Set The ID*/
	  clientNumber = idField.getText(); }
	else if ("Party".equals(obj)) {
	  /*Set The ID*/
	  partyName=partyField.getText();
	  binder();
	  System.out.println("Trying for creation of Party ....");

	  long startTime = System.currentTimeMillis();

	  if (partyCoordinator.createParty(partyName))
	    System.out.println("New Party created");
	  else 
	    System.out.println("Party being joined");
	  long stopTime = System.currentTimeMillis();
	  System.out.println("Avg Ping to invoke = "
			 + ((stopTime - startTime)/1000f) + " msecs");
	  
	  partyID= partyCoordinator.getPartyID(partyName);
	  System.out.println("Party Id is: "+partyID);

	  myCoordinator = partyCoordinator.getPartyHandle(partyID);
	  System.out.println("Received Party Handle");
	  return true;
	}
	
	else if ("Set Name".equals(obj)) {
	  
	  /* Set the name */
	  
	  clientName = nameField.getText();
	  if (clientNumber == null || clientName==null || partyName==null) {
	    nameField.setText("Need to give a name + ID before \"Set Name\"");
	    return(true);
	  } else {
	    connected = connect();
	    nameField.setEditable(false);
	    setNameButton.disable();
	    signOffButton.enable();
	    sayButton.enable();
	  }
	  return(true);
	} 
        return(false);
    }
}

/*This thread is responsible for the callbacks from the Server side */

class ClientControlThread extends Thread
{
  private ChatUser thisClient;
  private MultiCoordinator.Coordinator myCoordinator;
  private com.visigenic.vbroker.IOP.IORHelper remote_ior_file; 
  private PartyScheduler.PartyCoordinator partyCoordinator;
  ClientControlThread(ChatUser client)
  { thisClient = client;
  }

  public void run()
  { try
    { // Initialize the ORB.
      System.out.println("trying to initialize the ORB..");
      org.omg.CORBA.ORB orb = org.omg.CORBA.ORB.init(thisClient);
      
      // Initialize the BOA.
      System.out.println("trying to initialize the BOA...");
      org.omg.CORBA.BOA boa = orb.BOA_init();
      

      // Create the ClientControl object.
      ClientControlImpl control =
                 new ClientControlImpl(thisClient);

      // Export the newly created object.
      boa.obj_is_ready(control);


      System.out.println("Registering ......");
      // Register with Coordinator
           
      int id= thisClient.myCoordinator.register(thisClient.clientNumber.hashCode(), thisClient.clientName,(Client.ClientControl) control);
      thisClient.clientID=id;
      if (id < 20)

	{
          System.out.println("Registration Succeeded");
	}
      else
	{
	   System.out.println("Registration Failed");
	}
      
      // Ready to service requests.
      System.out.println("Control: Waiting for requests");
      //boa.impl_is_ready();
    }
    catch(org.omg.CORBA.SystemException e)
    { System.err.println(e);
    }
   }
}
